package hdm.mi.growbros.auth;

import hdm.mi.growbros.exceptions.EmailAlreadyExistsException;
import hdm.mi.growbros.exceptions.InvalidDataException;
import hdm.mi.growbros.exceptions.InvalidEmailException;
import hdm.mi.growbros.exceptions.UserNotFoundException;
import hdm.mi.growbros.models.user.Role;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.UserRepository;
import hdm.mi.growbros.security.JwtService;
import lombok.RequiredArgsConstructor;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.NoSuchElementException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
@RequiredArgsConstructor
public class AuthenticationService {
    private final UserRepository repository;
    private final PasswordEncoder passwordEncoder;
    private final JwtService jwtService;
    private final AuthenticationManager authenticationManager;

    /**
     * Ermöglicht es einen User zu erstellen und diesen in der Datenbank zu registrieren
     * und daraus den Token zu erstellen.
     * @param request
     * @return
     */
    public AuthenticationResponse register(RegisterRequest request) {

        if (!isValidEmail(request.getEmail())) {
            throw new InvalidEmailException("Ungültige E-Mail-Adresse.");
        }

        if (request.getFirstname().isEmpty()) {
            throw new InvalidDataException("Vorname fehlt.");
        }

        if (request.getLastname().isEmpty()) {
            throw new InvalidDataException("Nachname fehlt.");
        }
        try {
            var user = User.builder()
                    .firstname(request.getFirstname())
                    .lastname(request.getLastname())
                    .email(request.getEmail())
                    .password(passwordEncoder.encode(request.getPassword()))
                    .role(Role.USER)
                    .build();
            repository.save(user);
            var jwtToken = jwtService.generateToken(user);
            return AuthenticationResponse.builder()
                    .token(jwtToken)
                    .build();
        } catch (DataIntegrityViolationException e) {
            throw new EmailAlreadyExistsException("Email ist bereits registriert.");
        }
    }

    public AuthenticationResponse authenticate(AuthenticationRequest request) {
        try {
            authenticationManager.authenticate(
                    new UsernamePasswordAuthenticationToken(
                            request.getEmail(),
                            request.getPassword()
                    )
            );
            var user = repository.findByEmail(request.getEmail()) //wenn der Nutzername und Passwort korrekt sind wird ein Token generiert
                    .orElseThrow(); // = Methode von Optional -> wird auf das Optionale angewendet, das durch repository.findByEmail zurückgegeben wird
            // Wenn Benutzer gefunden ->  das Optional mit dem Benutzer-Objekt gefüllt. Andernfalls wird eine NoSuchElementException ausgelöst.
            var jwtToken = jwtService.generateToken(user);
            return AuthenticationResponse.builder()
                    .token(jwtToken)
                    .build();
        } catch (NoSuchElementException e) {
            throw new UserNotFoundException("Benutzer mit Email:" + request.getEmail() + " nicht gefunden.");
        } catch (BadCredentialsException e) {
            throw new UserNotFoundException("Anmeldedaten sind nicht korrekt.");
        }
    }

    private boolean isValidEmail(String email) {
        String emailRegex = "^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Z|a-z]{2,}$";
        Pattern pattern = Pattern.compile(emailRegex);
        Matcher matcher = pattern.matcher(email);
        return matcher.matches();
    }

}
