package hdm.mi.growbros.controllers;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.service.GardenService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/v1/garden")
@RequiredArgsConstructor
public class GardenController {
    private static final Logger log = LoggerFactory.getLogger(GardenService.class);

    private final GardenService gardenService;

    @GetMapping
    @Operation(description = "Get plants in the garden")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved plants")
    })
    public ResponseEntity<List<Plant>> getPlantsInGarden(
            @RequestParam(value = "sort", required = false) String sort,
            @AuthenticationPrincipal User user
    ) {
        return ResponseEntity.ok(gardenService.getUserPlants(user, sort));
    }

    @GetMapping("/count")
    @Operation(description = "Get the size of the garden")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved garden size"),
    })
    public ResponseEntity<Integer> getGardenSize(
            @AuthenticationPrincipal User user
    ) {
        Integer response = gardenService.countByUser(user);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @PostMapping("/add/{plantId}")
    @Operation(description = "Add a plant to the garden")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "201", description = "Successfully added plant to the garden"),
            @ApiResponse(responseCode = "404", description = "Plant not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<Plant> addPlantToGarden(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        final Plant plantAdded = gardenService.addPlantToGarden(plantId, user);
        return ResponseEntity.status(201).body(plantAdded);
    }

    @DeleteMapping("/remove/{plantId}")
    @Operation(description = "Remove a plant from the garden")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Successfully removed plant from the garden"),
    })
    public ResponseEntity<Void> removePlantFromGarden(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        gardenService.removeFromGarden(plantId, user);
        return ResponseEntity.status(204).build();
    }

    @DeleteMapping("/remove/all")
    @Operation(description = "Remove all plants from the garden")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully removed all plants from the garden"),
    })
    public ResponseEntity<Integer> removeAllPlantsFromGarden(
            @AuthenticationPrincipal User user
    ) {
        Integer response = gardenService.clearGarden(user);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @PostMapping("/setNewWateringFrequency")
    @Operation(description = "Set new watering frequency for a plant")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Successfully set new watering frequency"),
    })
    public ResponseEntity<Void> setNewWateringService(
            @RequestParam("wateringFrequency") int wateringFrequency, @RequestBody GardenEntry gardenEntry
    ) {
        gardenEntry.setWateringFrequency(wateringFrequency);
        return ResponseEntity.ok(null);
    }
}
