package hdm.mi.growbros.controllers;

import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.dto.DeletedPlantDto;
import hdm.mi.growbros.models.dto.PlantUpdateRequest;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.search.SearchRequest;
import hdm.mi.growbros.service.PlantsService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;

@RestController
@CrossOrigin
@RequestMapping("/api/v1/plants")
@RequiredArgsConstructor
public class PlantsController {
    private final PlantsService plantsService;

    @GetMapping
    @Operation(description = "Get all plants")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved plants"),
    })
    public ResponseEntity<CustomPageDto<Plant>> getAllPlants(
            @RequestParam(value = "page", defaultValue = "0") int page,
            @RequestParam(value = "pageSize", defaultValue = "25") int pageSize
    ) {
        CustomPageDto<Plant> response = plantsService.getPlants(page,pageSize);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @GetMapping("/{id}")
    @Operation(description = "Get a specific plant by ID")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved the plant"),
            @ApiResponse(responseCode = "404", description = "Plant not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<Plant> getPlant(
            @PathVariable long id
    ) {
        Plant response = plantsService.findById(id).orElseThrow(() -> new PlantNotFoundException(id));
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @GetMapping("/randomPlants/{numberOfPlants}")
    @Operation(description = "Get a collection of random plants")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved random plants"),
    })
    public ResponseEntity<Collection<Plant>> getRandomPlants(
            @PathVariable Integer numberOfPlants
    ) {
        Collection<Plant> response = plantsService.getRandomPlants(numberOfPlants);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @GetMapping("/search")
    @Operation(description = "Search for plants")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved searched plants")
    })
    public ResponseEntity<Collection<Plant>> getPlantsSearchedFor(
            @Valid SearchRequest searchRequest
    ) {
        Collection<Plant> response = plantsService.getPlantsSearchedFor(searchRequest);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @PostMapping
    @Operation(description = "Create a new plant")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Successfully created the plant"),
            @ApiResponse(responseCode = "400", description = "Bad Request", content = @Content(mediaType = "text/plain")),
})
    public ResponseEntity<Void> createPlant(
            @RequestBody @Valid Plant plant
    ) {
        plantsService.createPlant(plant);
        return ResponseEntity.ok(null);
    }

    @DeleteMapping("/{id}")
    @Operation(description = "Delete a plant by ID")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully deleted the plant"),
            @ApiResponse(responseCode = "404", description = "Plant not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<DeletedPlantDto> deletePlant(
            @PathVariable long id
    ) {
        DeletedPlantDto response = plantsService.deletePlant(id);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @PatchMapping("/{id}")
    @Operation(description = "Update a plant by ID")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully updated the plant"),
            @ApiResponse(responseCode = "400", description = "Bad Request", content = @Content(mediaType = "text/plain")),
            @ApiResponse(responseCode = "404", description = "Plant not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<Plant> updatePlant(
            @PathVariable long id,
            @RequestBody PlantUpdateRequest updateRequest
    ) {
        Plant response = plantsService.updatePlant(id, updateRequest);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }
}
