package hdm.mi.growbros.controllers;

import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.service.WishListService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;

@RestController
@CrossOrigin
@RequestMapping("/api/v1/wishlist")
@RequiredArgsConstructor
public class WishListController {

    private final WishListService wishListService;

    @GetMapping
    @Operation(description = "Get all plants in wishlist")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved wishlist plants"),
    })
    public ResponseEntity<Collection<Plant>> getAllPlantsInWishList(
            @RequestParam(value = "sort", required = false) String sort,
            @AuthenticationPrincipal User user
    ) {
        Collection<Plant> response = wishListService.getWishedPlants(user, sort);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @GetMapping("/count")
    @Operation(description = "Get the size of the wishlist")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved wishlist size"),
    })
    public ResponseEntity<Integer> getWishListSize(
            @AuthenticationPrincipal User user
    ) {
        Integer response = wishListService.countByUser(user);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @PostMapping("/add/{plantId}")
    @Operation(description = "Add plant to wishlist")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "201", description = "Successfully added to wishlist"),
            @ApiResponse(responseCode = "404", description = "Plant not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<Void> addPlantToWishlist(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        wishListService.addPlantToWishList(plantId, user);
        return ResponseEntity.status(201).build();
    }

    @DeleteMapping("/remove/{plantId}")
    @Operation(description = "Remove plant from wishlist")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Successfully removed from wishlist"),
            @ApiResponse(responseCode = "404", description = "Wishlist entry not found", content = @Content(mediaType = "text/plain")),
    })
    public ResponseEntity<Void> removePlantFromWishlist(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        wishListService.removeFromWishList(plantId, user);
        return ResponseEntity.status(204).build();
    }

    @DeleteMapping("/remove/all")
    @Operation(description = "Remove all plants from wishlist")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully removed all from wishlist"),
    })
    public ResponseEntity<Integer> removeAllPlantsFromWishlist(
            @AuthenticationPrincipal User user
    ) {
        Integer response = wishListService.clearWishList(user);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }
}
