package hdm.mi.growbros.repositories;

import hdm.mi.growbros.models.WishListEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.Role;
import hdm.mi.growbros.models.user.User;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;

@DataJpaTest
@ActiveProfiles("test")
public class WishListRepositoryTest {

    @Autowired
    private WishListRepository underTest;

    @Autowired
    private TestEntityManager entityManager;

    private WishListEntry we1, we2, we3, we4;
    private User user1, user2;

    @BeforeEach
    void setupEach() {
        user1 = User.builder()
                .email("user1@gmail.com")
                .role(Role.USER)
                .password("safePassword123!")
                .build();
        entityManager.persist(user1);

        user2 = User.builder()
                .email("user2@gmail.com")
                .role(Role.USER)
                .password("safePassword123!")
                .build();
        entityManager.persist(user2);

        Plant p1 = Plant.builder()
                .name("Early plant")
                .plantWeekStart(1)
                .plantWeekEnd(3)
                .harvestWeekStart(10)
                .harvestWeekEnd(15)
                .build();
        entityManager.persist(p1);
        Plant p2 = Plant.builder()
                .name("Medium late plant")
                .plantWeekStart(4)
                .plantWeekEnd(6)
                .harvestWeekStart(18)
                .harvestWeekEnd(20)
                .build();
        entityManager.persist(p2);
        Plant p3 = Plant.builder()
                .name("Late plant")
                .plantWeekStart(10)
                .plantWeekEnd(15)
                .harvestWeekStart(25)
                .harvestWeekEnd(30)
                .build();
        entityManager.persist(p3);

        we1 = WishListEntry.builder().id(0L).plant(p1).user(user1).build();
        we2 = WishListEntry.builder().id(0L).plant(p2).user(user1).build();
        we3 = WishListEntry.builder().id(0L).plant(p3).user(user1).build();
        we4 = WishListEntry.builder().id(0L).plant(p3).user(user2).build();
        underTest.saveAll(List.of(we1, we2, we3, we4));
    }

    @Test
    void shouldReturnCorrectList_ofEntriesForUser_orderedByNearestPlantingWeek() {
        //when
        var result = underTest.findAllByNearestPlantingWeek(user1, 1);

        //expect
        assertEquals(List.of(we1, we2, we3), result);
    }

    @Test
    void shouldReturnCorrectList_ofEntriesForUser_orderedByCurrentPlantingWeek() {
        //when
        var result = underTest.findByCurrentPlantingWeek(user1, 1);

        //expect
        assertEquals(List.of(we1), result);
    }

    @Test
    void shouldDeleteCorrectEntries_whenClearingGarden_forUser() {
        //when
        int deleted = underTest.deleteAllByUser(user2);

        //expect
        assertEquals(1, deleted);
        assertEquals(List.of(), underTest.findByUser(user2));
        assertThat(List.of(we1, we2, we3)).hasSameElementsAs(underTest.findByUser(user1));
    }

    @Test
    void countShouldBeCorrect_forUser() {
        //when
        int user1Count = underTest.countByUser(user1);
        int user2Count = underTest.countByUser(user2);

        //expect
        assertEquals(3, user1Count);
        assertEquals(1, user2Count);
    }
}
